/*
/*  ChartBuilder.m
/*
 IMPORTANT:  This sample software is supplied to you by Blacksmith Technologies, Inc. ("Blacksmith") in consideration of your agreement to the following terms.  Your use, installation,  modification or redistribution of this software constitutes acceptance of these terms.  If you do not agree with these terms, please do not use, install, modify or redistribute this software.
  
This software is provided by Blacksmith on an "AS IS" basis.  Blacksmith makes no warranties, express or implied, including without limitation the implied warranties of non-infringement, merchantability and fitness for a particular purpose, regarding this software or its use and operation alone or in combination with your products.  In no event shall Blacksmith be liable for any special, indirect, incidental or consequential damages arising in any way out of the use, reproduction, modification and/or distribution of this software, however caused and whether under theory of contract, tort, strict liability or otherwise, even if Blacksmith has been advised of the possibility of such damage.

*/


#import "ChartBuilder.h"
#import "ChartsmithInterface.h"
#import "DataManager.h"


@implementation ChartBuilder

- (void)initializeForChartBuilding
{
    /* launch Chartsmith, create a document, then hide Chartsmith */
    ChartsmithInterface *interface = [ChartsmithInterface sharedChartsmithInterface];
    if( ![interface isChartsmithLaunched] ) [interface launchChartsmith];
    [interface createDocumentIfNecessary];
    [interface hideChartsmith];
}

- (void)buildPriceChartFromDataSet:(NSArray*)dataArray  atChartIndex:(int)chartIndex
{
    /* this method constructs the price chart based on the data that is passed in and a stored template */
    int i;
	NSString *path = [[NSBundle bundleForClass:[self class]] pathForResource:@"PriceData" ofType:@"cstemplate"];
    ChartsmithInterface *interface = [ChartsmithInterface sharedChartsmithInterface];
    NSMutableString *rowLabels = [NSMutableString string];
    NSMutableString *dataString = [NSMutableString string];
    
    /* be sure Chartsmith is running and we have a document */
    [self initializeForChartBuilding];
    
    /* create a high-low chart for price data, then apply a template to format the chart */
    [interface createChartOfType:@"hilo" atIndex:chartIndex];
    if( path )
        [interface applyTemplateAtPath:path toChartAtIndex:chartIndex];
    else 
        NSRunInformationalAlertPanel(@"Template Not Found",
            @"Charts will not be properly formatted. Copy 'PriceData.cstemplate' into Resources folder within application",@"OK",@"",nil);

    /* add as many rows as we have data elements in our data set, minus one 
    /* because one row is already created by default when the chart is created */
	[interface addRows:[dataArray count]-1 atIndex:1 toChartAtIndex:chartIndex];

    /* construct row label string and data string - these will be sent to Chartsmith
    /* each in a single command so that all row labels and all data are each set in a single call */
    for( i=0; i<[dataArray count]; i++ )
    {
        [rowLabels appendString:[[dataArray objectAtIndex:i] objectForKey:KEY_DATE]];
        if( i!=[dataArray count]-1 ) [rowLabels appendString:@";"]; // row delimiter = ';'
        
        [dataString appendFormat:@"%.2f,%.2f",[[[dataArray objectAtIndex:i] objectForKey:KEY_HIGH] floatValue],
             [[[dataArray objectAtIndex:i] objectForKey:KEY_LOW] floatValue]];   // col delimiter = ','
        if( i!=[dataArray count]-1 ) [dataString appendString:@";"];             // row delimiter = ';'
    }
    
    /* send data to Chartsmith */
	[interface setRowLabels:rowLabels ofChartAtIndex:chartIndex];
    [interface setDataValues:dataString ofChartAtIndex:chartIndex];
}

- (void)buildVolumeChartFromDataSet:(NSArray*)dataArray atChartIndex:(int)chartIndex
{
    int i;
    ChartsmithInterface *interface = [ChartsmithInterface sharedChartsmithInterface];
	NSString *path = [[NSBundle bundleForClass:[self class]] pathForResource:@"VolumeData" ofType:@"cstemplate"];
    NSMutableString *rowLabels = [NSMutableString string];
    NSMutableString *dataString = [NSMutableString string];
    
    /* be sure Chartsmith is running and we have a document */
    [self initializeForChartBuilding];

    /* create a bar chart for volume data, then apply a template to format the chart */
    [interface createChartOfType:@"bar" atIndex:chartIndex];
    if( path )
        [interface applyTemplateAtPath:path toChartAtIndex:chartIndex];
    else
        NSRunInformationalAlertPanel(@"Template Not Found",
            @"Charts will not be properly formatted. Copy 'VolumeData.cstemplate' into Resources folder within application",@"OK",@"",nil);
    
    /* add as many rows as we have data elements in our data set, minus one 
    /* because one row is already created by default when the chart is created */
	[interface addRows:[dataArray count]-1 atIndex:1 toChartAtIndex:chartIndex];

    /* construct row label string and data string - these will be sent to Chartsmith
    /* each in a single command so that all row labels and all data are each set in a single call */
    for( i=0; i<[dataArray count]; i++ )
    {
        [rowLabels appendString:[[dataArray objectAtIndex:i] objectForKey:KEY_DATE]];
        if( i!=[dataArray count]-1 ) [rowLabels appendString:@";"]; // row delimiter = ';'
        
        [dataString appendFormat:@"%.0f",[[[dataArray objectAtIndex:i] objectForKey:KEY_VOLUME] floatValue]];
        if( i!=[dataArray count]-1 ) [dataString appendString:@";"]; // row delimiter = ';'
    }

    /* send data to Chartsmith */
	[interface setRowLabels:rowLabels ofChartAtIndex:chartIndex];
    [interface setDataValues:dataString ofChartAtIndex:chartIndex];
}

- (void)setChartTitle:(NSString*)aTitle subTitle:(NSString*)aSubTitle forChart:(int)chartIndex
{
    ChartsmithInterface *interface = [ChartsmithInterface sharedChartsmithInterface];
	[interface setTitle:aTitle toChartAtIndex:chartIndex];
	[interface setSubTitle:aSubTitle toChartAtIndex:chartIndex];
}

- (void)exportChartToPath:(NSString*)path forChart:(int)chartIndex
{
    ChartsmithInterface *interface = [ChartsmithInterface sharedChartsmithInterface];
    [interface exportChartAtIndex:chartIndex toPath:path];
}


@end
