/*
/*  ChartsmithInterface.m
/*
 IMPORTANT:  This sample software is supplied to you by Blacksmith Technologies, Inc. ("Blacksmith") in consideration of your agreement to the following terms.  Your use, installation,  modification or redistribution of this software constitutes acceptance of these terms.  If you do not agree with these terms, please do not use, install, modify or redistribute this software.
  
This software is provided by Blacksmith on an "AS IS" basis.  Blacksmith makes no warranties, express or implied, including without limitation the implied warranties of non-infringement, merchantability and fitness for a particular purpose, regarding this software or its use and operation alone or in combination with your products.  In no event shall Blacksmith be liable for any special, indirect, incidental or consequential damages arising in any way out of the use, reproduction, modification and/or distribution of this software, however caused and whether under theory of contract, tort, strict liability or otherwise, even if Blacksmith has been advised of the possibility of such damage.

*/


#import "ChartsmithInterface.h"
#import <AppKit/AppKit.h>

static ChartsmithInterface  *sharedInstance;

@implementation ChartsmithInterface

+ sharedChartsmithInterface
{
    if( !sharedInstance )
        sharedInstance = [[ChartsmithInterface alloc] init];
    return sharedInstance;
}

- (NSString*)runScript:(NSString*)scriptText
{
    NSAppleScript *script;
	NSAppleEventDescriptor *result;
    NSMutableDictionary *errorInfo = [NSMutableDictionary dictionary];

    script = [[[NSAppleScript alloc] initWithSource:scriptText] autorelease];
    result = [script executeAndReturnError:&errorInfo];
    
    // uncomment the line below if you would like to view the AppleScript that is generated
    // printf("%s\n----------------------------------------\n",[scriptText cString]);
    
    if( !result ){
        printf("Error: %s\n",[[errorInfo objectForKey:NSAppleScriptErrorMessage] cString]);
        return nil;
    }
    return [result stringValue];

}

- (BOOL)isChartsmithLaunched
{
	NSArray *runningApps = [[NSWorkspace sharedWorkspace] launchedApplications];
    int i;
    
    for( i=0; i<[runningApps count]; i++ )
    {
        NSDictionary *appInfoDict = [runningApps objectAtIndex:i];
        NSString *runningApp = [appInfoDict objectForKey:@"NSApplicationName"];
        if( [runningApp isEqualToString:@"Chartsmith"] )
            return YES;
    }
    return NO;
}

- (void)launchChartsmith
{
	NSString *scriptText;
     
    scriptText = @"tell application \"Chartsmith\"\nActivate\nend tell";
    [self runScript:scriptText];
}

- (void)hideChartsmith
{
	NSString *scriptText;
    
    scriptText = @"tell application \"Finder\"\nset visible of process \"Chartsmith\" to false\nend tell";
	[self runScript:scriptText];
}

- (BOOL)isChartsmithLicensed
{
	NSString *scriptText;
	NSString *resultText;
    
    scriptText = @"tell application \"Chartsmith\"\nvalid license\nend tell";
	resultText = [self runScript:scriptText];
    if( [resultText isEqualToString:@"YES"] )
        return YES;
    return NO;
}

- (void)createDocumentIfNecessary
{
	NSString *scriptText;

    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\nif not (document 1 exists) then\nmake new document at beginning of documents\nend if\nend tell"];
    [self runScript:scriptText];
}

- (void)createChartOfType:(NSString*)type atIndex:(int)chartIndex
{
	NSString *scriptText;
     
    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\nadd chart of type %@ at index %d\nend tell\nend tell",type,chartIndex];
    [self runScript:scriptText];
}

- (void)addRowAtIndex:(int)rowIndex toChartAtIndex:(int)chartIndex
{
	NSString *scriptText;
     
    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell datasheet of chart %d\ninsert row at index %d\nend tell\nend tell\nend tell",chartIndex,rowIndex];
    [self runScript:scriptText];
}
    
- (void)addRows:(int)count atIndex:(int)rowIndex toChartAtIndex:(int)chartIndex
{
	NSString *scriptText;
     
    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell datasheet of chart %d\ninsert rows count %d at index %d\nend tell\nend tell\nend tell",chartIndex,count,rowIndex];
    [self runScript:scriptText];
}
    
- (void)setRowLabel:(NSString*)label atRowIndex:(int)rowIndex ofChartAtIndex:(int)chartIndex
{
	NSString *scriptText;

    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell datasheet of chart %d\nset row label for row %d to \"%@\"\nend tell\nend tell\nend tell",chartIndex,rowIndex,label];
    [self runScript:scriptText];
}

- (void)setRowLabels:(NSString*)labelsString ofChartAtIndex:(int)chartIndex
{
	NSString *scriptText;

    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell datasheet of chart %d\nset row labels to \"%@\"\nend tell\nend tell\nend tell",chartIndex,labelsString];
    [self runScript:scriptText];
}

- (void)setDataValue:(float)value forRow:(int)row column:(int)column subcolumn:(int)subcolumn ofChartAtIndex:(int)chartIndex
{
	NSString *scriptText;

    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell datasheet of chart %d\nset cell row %d col %d subcol %d to %.2f\nend tell\nend tell\nend tell",chartIndex,row,column,subcolumn,value];
    [self runScript:scriptText];
}

- (void)setDataValues:(NSString*)dataString ofChartAtIndex:(int)chartIndex
{
	NSString *scriptText;

    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell datasheet of chart %d\nset cells to \"%@\"\nend tell\nend tell\nend tell",chartIndex,dataString];
    [self runScript:scriptText];
}

- (void)applyTemplateAtPath:(NSString*)path toChartAtIndex:(int)chartIndex
{
	NSString *scriptText;

    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell chart %d\napply template \"%@\"\nend tell\nend tell\nend tell",chartIndex,path];
    [self runScript:scriptText];
}

- (void)setTitle:(NSString*)aTitle toChartAtIndex:(int)chartIndex
{
	NSString *scriptText;

    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell chart %d\nset title to \"%@\"\nend tell\nend tell\nend tell",chartIndex,aTitle];
    [self runScript:scriptText];
}

- (void)setSubTitle:(NSString*)aTitle toChartAtIndex:(int)chartIndex
{
	NSString *scriptText;

    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell chart %d\nset subtitle to \"%@\"\nend tell\nend tell\nend tell",chartIndex,aTitle];
    [self runScript:scriptText];
}

- (void)exportChartAtIndex:(int)chartIndex toPath:(NSString*)path
{
	NSString *scriptText;

    scriptText = [NSString stringWithFormat:@"tell application \"Chartsmith\"\ntell first document\ntell chart %d\nexport as \"PDF\" to \"%@\"\nend tell\nend tell\nend tell",chartIndex,path];
    [self runScript:scriptText];
}


@end
