/*
/*  DataManager.m
/*
 IMPORTANT:  This sample software is supplied to you by Blacksmith Technologies, Inc. ("Blacksmith") in consideration of your agreement to the following terms.  Your use, installation,  modification or redistribution of this software constitutes acceptance of these terms.  If you do not agree with these terms, please do not use, install, modify or redistribute this software.
  
This software is provided by Blacksmith on an "AS IS" basis.  Blacksmith makes no warranties, express or implied, including without limitation the implied warranties of non-infringement, merchantability and fitness for a particular purpose, regarding this software or its use and operation alone or in combination with your products.  In no event shall Blacksmith be liable for any special, indirect, incidental or consequential damages arising in any way out of the use, reproduction, modification and/or distribution of this software, however caused and whether under theory of contract, tort, strict liability or otherwise, even if Blacksmith has been advised of the possibility of such damage.

*/

#import "DataManager.h"
#import <SystemConfiguration/SystemConfiguration.h>

/* SystemConfiguration strings */
#define BFSCGlobalIPv4StateKey CFSTR("State:/Network/Global/IPv4")
#define BFSCRouterKey          (NSString*)kSCPropNetIPv4Router


@implementation DataManager

- (NSString *)appName
{
    return [[NSProcessInfo processInfo] processName];
}

- (NSURL*)urlForSymbol:(NSString*)symbol date1:(NSCalendarDate*)date1 date2:(NSCalendarDate*)date2
{
    /* build URL to Yahoo's historical data repository */
    NSMutableString *url = [NSMutableString stringWithString:@"http://chart.yahoo.com/table.csv?"];
    
    [url appendFormat:@"a=%d&b=%d&c=%d&d=%d&e=%d&f=%d&s=%@&y=0&g=d&ignore=.csv",
        [date1 monthOfYear]-1,
        [date1 dayOfMonth]-1,
        [date1 yearOfCommonEra],
        [date2 monthOfYear]-1,
        [date2 dayOfMonth]-1,
        [date2 yearOfCommonEra],
        symbol];
    
    return [NSURL URLWithString:url];
}

- (BOOL)isNetworkConnectionAvailable
{
    NSDictionary *ipv4Dict;
    SCDynamicStoreRef store;
    BOOL connected = NO;

    store = SCDynamicStoreCreate(NULL, (CFStringRef)[self appName], NULL, NULL);
    if (store == NULL) {
        return NO;
    }

    ipv4Dict = (id)SCDynamicStoreCopyValue(store, BFSCGlobalIPv4StateKey);

    if (ipv4Dict != nil)
    {
        /* if we don't have a router, assume we aren't connected */
        if ([ipv4Dict isKindOfClass:[NSDictionary class]] &&
            [ipv4Dict objectForKey:BFSCRouterKey] != nil)
        {
            connected = YES;
        }

        CFRelease(ipv4Dict);
    }

    CFRelease(store);
    return connected;
}

- (NSArray*)parseData:(NSMutableString*)data endingAtDate:(NSCalendarDate*)lastDateRead
{
    NSMutableArray *resultSet = [NSMutableArray array];
    NSScanner *scanner = [NSScanner scannerWithString:data];
    NSRange range = [data rangeOfString:@"\n"];
    NSMutableString *delimiter = [NSMutableString string];
    NSCalendarDate  *testDate = nil;
    NSRange dashRange, numberRange, illegalRange;
    
    /* these _seem_ to be the conditions that result from 
    /* erroneous data - your mileage may vary             */
    dashRange = [data rangeOfString:@"-"];
    numberRange = [data rangeOfCharacterFromSet:[NSCharacterSet decimalDigitCharacterSet]];
    numberRange = [data rangeOfCharacterFromSet:[NSCharacterSet decimalDigitCharacterSet]];
    illegalRange = [data rangeOfCharacterFromSet:[NSCharacterSet illegalCharacterSet]];
    
    if( !data || 
        ![data length] || 
        [data length]==64 || 
        dashRange.length==0 || 
        numberRange.length==0 || 
        illegalRange.length!=0 || 
        [data isEqualToString:@"Date,Open,High,Low,Close,Volume\n"] )
        return nil;
        
    [scanner setScanLocation:range.location+range.length];

    while( ![scanner isAtEnd] && ![lastDateRead isEqualToDate:testDate] )
    {
        NSMutableDictionary *oneRowOfData = [NSMutableDictionary dictionary];
        NSMutableString *date = [NSMutableString string];
        NSMutableString *open = [NSMutableString string];
        NSMutableString *high = [NSMutableString string];
        NSMutableString *low = [NSMutableString string];
        NSMutableString *close = [NSMutableString string];
        NSMutableString *volume = [NSMutableString string];

        [scanner scanUpToString:@"," intoString:&date];
        [scanner scanString:@"," intoString:&delimiter];
        [oneRowOfData setObject:date forKey:KEY_DATE];
        testDate = [NSCalendarDate dateWithString:date calendarFormat:@"%d-%b-%y"];
        
        [scanner scanUpToString:@"," intoString:&open];
        [scanner scanString:@"," intoString:&delimiter];
        [oneRowOfData setObject:open forKey:KEY_OPEN];
    
        [scanner scanUpToString:@"," intoString:&high];
        [scanner scanString:@"," intoString:&delimiter];
        [oneRowOfData setObject:high forKey:KEY_HIGH];
    
        [scanner scanUpToString:@"," intoString:&low];
        [scanner scanString:@"," intoString:&delimiter];
        [oneRowOfData setObject:low forKey:KEY_LOW];
    
        [scanner scanUpToString:@"," intoString:&close];
        [scanner scanString:@"," intoString:&delimiter];
        [oneRowOfData setObject:close forKey:KEY_CLOSE];
    
        [scanner scanUpToString:@"\n" intoString:&volume];
        [scanner scanString:@"\n" intoString:&delimiter];
        [oneRowOfData setObject:volume forKey:KEY_VOLUME];
        
        /* insert at the beginning since data is received in reverse chronological order */
        [resultSet insertObject:oneRowOfData atIndex:0];
    }
    return resultSet;
}

- (NSArray*)retrieveDataForSymbol:(NSString*)symbol fromDate:(NSCalendarDate*)fromDate toDate:(NSCalendarDate*)toDate
{
    NSArray *dataSet;
    
    if (![self isNetworkConnectionAvailable])
        return nil;
    
    NS_DURING {
        NSURL  *url = [self urlForSymbol:symbol date1:fromDate date2:toDate];
        NSData *data = [url resourceDataUsingCache:NO];
       	NSMutableString *dataString = [NSMutableString stringWithCString:[data bytes]];
        dataSet = [self parseData:dataString endingAtDate:fromDate];
    } NS_HANDLER {
        NS_VALUERETURN(nil,NSArray*);
    } NS_ENDHANDLER;

    return dataSet;
}



@end
